package com.putnam.jboss.issues;

import java.util.Hashtable;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageProducer;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.naming.Context;
import javax.naming.InitialContext;

public class SelectorTest {
    private final static String HQ_JNDI_FACTORY ="org.jnp.interfaces.NamingContextFactory";
    private final static String HQ_PROVIDER_URL = "jnp://richmond1.putnaminv.com:6142";
    private final static String HQ_JNDI_PRINCIPAL = "";
    private final static String HQ_JNDI_PASSWORD = "";
    private final static String HQ_JNDI_URL_PKGS = "org.jboss.naming:org.jnp.interfaces";
    public final static String HQ_JMX_SERVICE_URL = "service:jmx:rmi:///jndi/rmi://richmond1:7142/jmxrmi"; 
    public static Hashtable<String, String> HQEnv = new Hashtable<String, String>();
    static {
        HQEnv.put(Context.INITIAL_CONTEXT_FACTORY, HQ_JNDI_FACTORY);
        HQEnv.put(Context.PROVIDER_URL, HQ_PROVIDER_URL);
        HQEnv.put(Context.SECURITY_PRINCIPAL, HQ_JNDI_PRINCIPAL);
        HQEnv.put(Context.SECURITY_CREDENTIALS, HQ_JNDI_PASSWORD); 
        HQEnv.put(Context.URL_PKG_PREFIXES, HQ_JNDI_URL_PKGS);
    }
    
    /**
     * @param args
     * @throws Exception
     */
    public static void main(String[] args) throws Exception {
        SelectorTest client = new SelectorTest();
        client.run(args);
    }
    
    public void run(String[] args) {
        ConnectionFactory connFactory = null;
        Connection conn = null;
        Session session = null;
        MessageProducer msgProducer = null;
        MessageConsumer msgConsumer = null;
        Destination aDestination = null;
        Message msg = null;;
        TextMessage tm = null;
        
        try {
            InitialContext ctx = new InitialContext(HQEnv);            
            connFactory = (ConnectionFactory) ctx.lookup("/ConnectionFactory");
            conn = connFactory.createConnection("guest", "guest");
            conn.setClientID("JBossPOC.SelectorTest");
            session = conn.createSession(false, Session.CLIENT_ACKNOWLEDGE);
            aDestination = (Destination) ctx.lookup("/queue/ExampleQueue");
            msgProducer = session.createProducer(aDestination);
            System.out.println("Using destination: [" + "/queue/ExampleQueue" + "]");
            conn.start();
            
            /* Publish messages */
            tm = session.createTextMessage();
            tm.setText("1");
            tm.setStringProperty("PROP1", "VALUE1");
            msgProducer.send(tm);
            System.out.println ("Sent message with id [" + tm.getJMSMessageID() + "]");

            tm = session.createTextMessage();
            tm.setText("2");
            tm.setStringProperty("PROP1", "VALUE1");
            msgProducer.send(tm);
            System.out.println ("Sent message with id [" + tm.getJMSMessageID() + "]");

            tm = session.createTextMessage();
            tm.setText("3");
            tm.setStringProperty("PROP2", "VALUE2");
            msgProducer.send(tm);
            System.out.println ("Sent message with id [" + tm.getJMSMessageID() + "]");

            tm = session.createTextMessage();
            tm.setText("4");
            tm.setStringProperty("PROP2", "VALUE2");
            msgProducer.send(tm);
            System.out.println ("Sent message with id [" + tm.getJMSMessageID() + "]");

            tm = session.createTextMessage();
            tm.setText("5");
            tm.setStringProperty("PROP1", "VALUE1");
            msgProducer.send(tm);
            System.out.println ("Sent message with id [" + tm.getJMSMessageID() + "]");

            tm = session.createTextMessage();
            tm.setText("6");
            tm.setStringProperty("PROP1", "VALUE1");
            tm.setStringProperty("PROP2", "VALUE2");
            msgProducer.send(tm);
            System.out.println ("Sent message with id [" + tm.getJMSMessageID() + "]");
            msgProducer.close();
            msgProducer = null;
            
            msgConsumer = session.createConsumer(aDestination, "PROP2 = 'VALUE2'");
            tm = (TextMessage) msgConsumer.receive(2*1000);
            if (tm == null) {
                System.err.println ("No message received");
            } else {
                tm.acknowledge();
                System.out.println ("Received message with id [" + tm.getJMSMessageID() + "]. Message body is [" + tm.getText() + "]");
            }
            
            /* close the consumer and re-open a different consumer with a null selector */
            msgConsumer.close();
            msgConsumer = null;
            session.close();
            conn.close();
            
            conn = connFactory.createConnection("guest", "guest");
            conn.start();
            session = conn.createSession(false, Session.CLIENT_ACKNOWLEDGE);
            msgConsumer = session.createConsumer(aDestination);
            tm = (TextMessage) msgConsumer.receive(2*1000);
            if (tm == null) {
                System.err.println ("No message received");
            } else {
                tm.acknowledge();
                System.out.println ("Received message with id [" + tm.getJMSMessageID() + "]. Message body is [" + tm.getText() + "]");
            }
            
        } catch (Exception e) {
            e.printStackTrace();
            System.err.println("An exception occurred");
            return;
        } finally {
            try {
                if (msgProducer != null) {
                    msgProducer.close();
                    msgProducer = null;
                    System.out.println("Closed producer...");
                }                
                if (msgConsumer != null) {
                    msgConsumer.close();
                    msgConsumer = null;
                    System.out.println("Closed consumer...");
                }
                if (session != null) {
                    session.close();
                    session = null;
                    System.out.println("Closed session...");
                }
                if (conn != null) {
                    conn.close();
                    conn = null;
                    System.out.println("Closed connection...");
                }
            } catch (JMSException jmse2) {
                jmse2.printStackTrace();
                System.err.println("An exception occurred when closing objects: " + jmse2.getMessage());
            }
        }
    }
}
